#!/usr/bin/env python3

from xml.etree import ElementTree
import sys
import os

LINE_TEMPLATE = "server     %name%      %host%:%port%       check"
TEMPLATE = None
HELP_TEXT = '''
haproxy-config.py [-p <pb-scp.cfg>] [-r <roleconfig.xml>] [-t <config-template>] [-o <output>]

    -p : pb-scp.cfg file path (default: /home/medsrv/var/dicom/pb-scp.cfg)
    -r : roleconfig.xml file path (default: /home/medsrv/var/conf/roleconfig.xml)
    -o : output file path (default: ./haproxy.cfg)
    -t : template file path (default: ./config.tpl)
    
'''

root = None
loop = None
form = dict()

default_roleconfig_path = "/home/medsrv/var/conf/roleconfig.xml"
default_haproxyconfig_path = "haproxy.cfg"
default_pbscpconfig_path = "/home/medsrv/var/dicom/pb-scp.cfg"
default_template_path = "config.tpl"
default_ip = '127.0.0.1'

ws_default_port = 8091
ws_name = "stm"
ws_list = []

ws_tls_default_port = 8093
ws_tls_list = []

web_default_port = 443
web_name = "web"
web_list = []

dicom_default_port = 102
dicom_name = "reg"
dicom_list = []

dicom_tls_default_port = 2762
dicom_tls_list = []


def fill_with_name(name):
    def fill_template(arg):
        return LINE_TEMPLATE\
            .replace("%name%", name + str(arg[0]))\
            .replace("%host%", str(arg[1][0]))\
            .replace("%port%", str(arg[1][1]))
    return fill_template


def save():
    web_filled = "\n\t".join(map(fill_with_name(web_name), enumerate(web_list)))
    ws_filled = "\n\t".join(map(fill_with_name(ws_name), enumerate(ws_list)))
    ws_tls_filled = "\n\t".join(map(fill_with_name(ws_name), enumerate(ws_tls_list)))
    dicom_filled = "\n\t".join(map(fill_with_name(dicom_name), enumerate(dicom_list)))
    dicom_tls_filled = "\n\t".join(map(fill_with_name(dicom_name), enumerate(dicom_tls_list)))

    filled_cfg = TEMPLATE\
        .replace("%dicom_tls%", dicom_tls_filled)\
        .replace("%dicom%", dicom_filled)\
        .replace("%websocket%", ws_filled)\
        .replace("%websocket_tls%", ws_tls_filled)\
        .replace("%web%", web_filled)\
        .replace("%meduser%", os.environ.get('MED_USER'))

    with open(default_haproxyconfig_path, 'w') as config:
        config.truncate(0)
        config.write(filled_cfg)


def load_config(roleconfig_path, pbscp_path, template):
    global root
    global dicom_default_port, dicom_tls_default_port
    global TEMPLATE

    TEMPLATE = open(template, 'r').read()

    with open(pbscp_path, "r") as cnf:
        p = list(filter(lambda line: line.startswith('PORT') or line.startswith('TLS_PORT'), cnf.readlines()))
        p = list(map(lambda line: line.replace("\"", ''), p))
        dicom_default_port = int(p[0].split('=')[1] if p[0].startswith('PORT') else p[1].split('=')[1])
        dicom_tls_default_port = int(p[0].split('=')[1] if p[0].startswith('TLS_') else p[1].split('=')[1])

    tree = ElementTree.parse(roleconfig_path)
    root = tree.getroot()
    for host in root:
        hostname = host.attrib['hostname']
        for role in host:
            t = role.attrib['type']
            if t == 'registration':
                dicom_list.append([hostname, dicom_default_port])
                dicom_tls_list.append([hostname, dicom_tls_default_port])
            if t == 'web':
                web_list.append([hostname, web_default_port])
            if t == 'stream':
                ws_list.append([hostname, ws_default_port])
                ws_tls_list.append([hostname, ws_tls_default_port])


if __name__ == "__main__":
    for idx in range(1, len(sys.argv), 2):
        if '-r' == sys.argv[idx]:
            default_roleconfig_path = sys.argv[idx+1]
        if '-o' == sys.argv[idx]:
            default_haproxyconfig_path = sys.argv[idx+1]
        if '-p' == sys.argv[idx]:
            default_pbscpconfig_path = sys.argv[idx+1]
        if '-t' == sys.argv[idx]:
            default_template_path = sys.argv[idx+1]
        if '-h' == sys.argv[idx]:
            print(HELP_TEXT)
            exit()

    load_config(default_roleconfig_path, default_pbscpconfig_path, default_template_path)
    save()
